//------------------------------------------------------
//
//   Greg's Blocks Mod - Main
//
//------------------------------------------------------

package gcewing.blocks;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.Map;
import java.util.Hashtable;

import net.minecraft.block.*;
import net.minecraft.item.*;

import net.minecraftforge.common.*;
import cpw.mods.fml.common.*;
import cpw.mods.fml.common.event.*;
import cpw.mods.fml.common.network.*;
import cpw.mods.fml.common.registry.*;

@Mod(modid="GregsBlocks", name="Greg's Blocks" , version="1.1.2")

@NetworkMod(clientSideRequired=true, serverSideRequired=false, versionBounds = "[1.1,1.2)",
	channels={PacketHandler.channelName}, packetHandler = PacketHandler.class)

public class GregsBlocks {

	@Mod.Instance("GregsBlocks")
	public static GregsBlocks mod;

	@SidedProxy(clientSide="gcewing.blocks.ProxyClient", serverSide="gcewing.blocks.Proxy")
	public static Proxy proxy;
	
	public static String modName = "GregsBlocks";
//	public static String channelName = "gce.blocks";
//	public static String version = "1.0";
	public final static String textureDir = "/gcewing/blocks/resources/";
	public final static String textureFile = textureDir + "textures.png";
	
	static String configName = modName + ".cfg";
	static File cfgfile;
	static OrderedProperties config;
	static Map <Integer, String> idToBlockName = new Hashtable <Integer, String>();
	static Map <Integer, String> idToItemName = new Hashtable <Integer, String>();
	static int nextBlockID = 1, nextItemID = 1;
	static boolean autoAssign;
	
//	public static boolean isServer;
	
	//
	//   Blocks and Items
	//
	
	public static BlockSawbench blockSawbench;
	public static Block blockShape;

	public static Item itemSawblade;
	public static Item itemLargePulley;
	public static Item itemChisel;
	
	@Mod.PreInit
	public void preInit(FMLPreInitializationEvent e) {
		cfgfile = e.getSuggestedConfigurationFile();
	}

	@Mod.Init
	public void load(FMLInitializationEvent e) {
		//System.out.printf("GregsBlocks: load()\n");
		NetworkRegistry.instance().registerGuiHandler(this, proxy);
		loadConfig();
		registerBlocks();
		registerTileEntities();
		registerItems();
		addRecipes();
//		registerChannels();
		proxy.load();
		saveConfig();
	}
	
	void loadConfig() {
		config = new OrderedProperties();
		try {
			config.load(new FileInputStream(cfgfile));
		}
		catch (FileNotFoundException e) {
			System.out.printf("%s: No existing config file\n", modName);
		}
		catch (IOException e) {
			System.out.printf("%s: Failed to read %s\n%s\n", modName,
				cfgfile.getPath(), e);
		}
		autoAssign = getConfigBool("autoAssign", false);
		//System.out.printf("%s: autoAssign = %s\n", modName, autoAssign);
		config.extended = false;
	}
	
	void saveConfig() {
		try {
			if (autoAssign || config.extended) {
				putConfigBool("autoAssign", false);
				System.out.printf("%s: Writing config file\n", modName);
				config.store(new FileOutputStream(cfgfile),
					modName + " Configuration File");
			}
		}
		catch (IOException e) {
			System.out.printf("%s: Failed to %s\n%s\n", modName,
				cfgfile.getPath(), e);
		}
	}

//	public String getVersion() {
//		return version;
//	}
	
//	void registerChannels() {
//		ModLoader.registerPacketChannel(this, channelName);
//	}
	
	void registerBlocks() {
		// Reserved range: 1040-1049
		//System.out.printf("%s: registerBlocks()\n", modName);
//		blockXXX = new BlockXXX(getBlockID(nnn, "XXX"));
//		blockXXX.setHardness(1.5F);
//		addBlock("X X X", blockXXX);
		blockSawbench = new BlockSawbench(getBlockID(1040, "sawbench"));
		blockSawbench.setHardness(2.0F);
		addBlock("Sawbench", blockSawbench);
		blockShape = addBlock("Shaped Block", new BlockShape(getBlockID(1041, "shape")), ItemShape.class);
	}
	
	void registerTileEntities() {
		GameRegistry.registerTileEntity(TESawbench.class, "gcewing.sawbench");
		GameRegistry.registerTileEntity(TEShape.class, "gcewing.shape");
	}
	
	void registerItems() {
		// Reserved range: 10400-10419
//		itemXXX = addItem("X X X", newItemI(nnnnn, "xxx", 0xNN));
		itemSawblade = addItem("Circular Saw Blade", newItemI(10400, "sawblade", 0x12));
		itemLargePulley = addItem("Large Pulley", newItemI(10401, "largePulley", 0x13));
		itemChisel = addItem("Chisel", new ItemChisel(getItemID(10402, "chisel")).setIconIndex(0x14));
	}

	void addRecipes() {
		//addShapelessRecipe(bonemealAndCharcoal, 1, new ItemStack(Item.dyePowder, 1, 15), new ItemStack(Item.coal, 1, 1));
		//addRecipe(floodlight, 1, "IrI", "IgI", "GGG",
		//	'I', Item.ingotIron, 'r', Item.redstone, 'g', Item.lightStoneDust, 'G', Block.glass);
		//addSmeltingRecipe(calciumCarbide, 1, bonemealAndCharcoal);
		addRecipe(blockSawbench, 1, "IsI", "WWP", "WWp",
			'I', Item.ingotIron, 's', itemSawblade, 'W', Block.planks, 'p', Block.pressurePlatePlanks,
			'P', itemLargePulley);
		addRecipe(itemSawblade, 1, " I ", "IsI", " I ",
			'I', Item.ingotIron, 's', Item.stick);
		addRecipe(itemLargePulley, 1, " W ", "WsW", " W ",
			'W', Block.planks, 's', Item.stick);
		addRecipe(itemChisel, 1, "I", "s",
			'I', Item.ingotIron, 's', Item.stick);
}
	
	void addRecipe(Item product, int qty, Object... params) {
		GameRegistry.addRecipe(new ItemStack(product, qty), params);
	}

	void addRecipe(Block product, int qty, Object... params) {
		GameRegistry.addRecipe(new ItemStack(product, qty), params);
	}

	void addShapelessRecipe(Item product, int qty, Object... params) {
		GameRegistry.addShapelessRecipe(new ItemStack(product, qty), params);
	}

	void addSmeltingRecipe(Item product, int qty, Item input) {
		GameRegistry.addSmelting(input.shiftedIndex, new ItemStack(product, qty), 0);
	}
	
	void addSmeltingRecipe(Item product, int qty, Block input) {
		GameRegistry.addSmelting(input.blockID, new ItemStack(product, qty), 0);
	}
	
	static int getBlockID(int defaultID, String name) {
		if (autoAssign)
			defaultID = nextUnusedBlockID();
		return getBlockOrItemID(defaultID, "tile.", name, idToBlockName);
	}
	
	static int getItemID(int defaultID, String name) {
		if (autoAssign)
			defaultID = nextUnusedItemID();
		return getBlockOrItemID(defaultID, "item.", name, idToItemName);
	}
	
	static int getBlockOrItemID(int defaultID, String prefix, String name, 
			Map <Integer, String> idToName)
	{
		String qualName = "gcewing." + name;
		String key = prefix + qualName + ".id";
		int id;
		if (config.containsKey(key)) {
			String value = (String) config.get(key);
			id = Integer.parseInt(value);
		}
		else {
			config.put(key, Integer.toString(defaultID));
			id = defaultID;
		}
		idToName.put(Integer.valueOf(id), qualName);
		return id;
	}
	
	static int nextUnusedBlockID() {
		//System.out.printf("%s: nextUnusedBlockID\n", modName);
		while (nextBlockID < 4096) {
			if (Block.blocksList[nextBlockID] == null)
				return nextBlockID;
			nextBlockID += 1;
		}
		throw new RuntimeException(modName + ": Out of block IDs");
	}
	
	static int nextUnusedItemID() {
		//System.out.printf("%s: nextUnusedItemID\n", modName);
		while (nextItemID < 32768) {
			if (Item.itemsList[nextItemID + 256] == null)
				return nextItemID;
			nextItemID += 1;
		}
		throw new RuntimeException(modName + ": Out of item IDs");
	}
	
	boolean getConfigBool(String key, boolean defaultValue) {
		if (config.containsKey(key))
			return ((String)config.get(key)).equals("true");
		else {
			putConfigBool(key, defaultValue);
			return defaultValue;
		}
	}
	
	void putConfigBool(String key, boolean value) {
		config.put(key, value ? "true" : "false");
	}

	Item newItem(int id, String name) {
		return customItem(getItemID(id, name));
	}
	
	Item newItemI(int id, String name, int icon) {
		return customItem(getItemID(id, name)).setIconIndex(icon);
	}
	
	Item newItemSI(int id, String name, int maxStack, int icon) {
		return customItem(getItemID(id, name)).setMaxStackSize(maxStack).setIconIndex(icon);
	}
	
	Item customItem(int id) {
		return new TexturedItem(id);
	}
		
	Block addBlock(String title, Block block) {
		return addBlock(title, block, ItemBlock.class);
	}

	Block addBlock(String title, Block block, Class itemClass) {
		//System.out.printf("%s: Adding block %s id %s\n", modName, title, block.blockID);
		block.setBlockName(idToBlockName.get(block.blockID));
		GameRegistry.registerBlock(block, itemClass);
		LanguageRegistry.addName(block, title);
		return block;
	}

	Item addItem(String title, Item item) {
		int id = item.shiftedIndex - 256;
		//System.out.printf("%s: Adding item %s id %s\n", modName, title, id);
		item.setItemName(idToItemName.get(id));
		LanguageRegistry.addName(item, title);
		return item;
	}
	
}
